button = {}

local function create(x,y,text,font,action,state)
	if x == "center" then
		local width = love.window.getWidth()
		x = width / 2 - font:getWidth( text ) / 2 + camera.x
	elseif x == "right" then
		local width = love.window.getWidth()
		x = width - font:getWidth( text ) + camera.x - 30
	end
	table.insert(button,{x=x,y=y,text=text,font=font,action=action,state=state,selected=false,hover=false})
end

function button_draw()
	for i,v in ipairs(button) do
		if v.state == gamestate then
			if v.hover then
				love.graphics.setColor(150,150,150)
			else
				love.graphics.setColor(0,0,0)
			end

			love.graphics.setFont(v.font)

			if v.selected and v.hover then
				love.graphics.rectangle("fill", v.x+camera.x, v.y+camera.y + v.font:getHeight() - v.font:getHeight()/20, v.font:getWidth(v.text), 2)
			end

			love.graphics.print(v.text,v.x+camera.x,v.y+camera.y)
			love.graphics.setColor(255,255,255)
		end
	end
end

function button_onClick(x,y)
	for i,v in ipairs(button) do
		if v.state == gamestate then
			if x > v.x and x < v.x + v.font:getWidth(v.text) and y > v.y and y < v.y + v.font:getHeight(v.text) then
				--love.audio.stop(effects.buttonClick)
				v.selected = true
				--effects.buttonClick:play()
			end
		end
	end
end

function button_onRelease(x,y)
	for i,v in ipairs(button) do
		if v.state == gamestate then
			if x > v.x and x < v.x + v.font:getWidth(v.text) and y > v.y and y < v.y + v.font:getHeight(v.text) then
				if v.selected then
					--love.audio.stop(effects.buttonRelease)
					v.action()
					v.selected = false --#noscope420blzet
					--effects.buttonRelease:play()
				end
			else
				v.selected = false --#cut4bieber
			end
		end
	end
end

function button_onHover(dt)
	for i, v in ipairs(button) do
		local mousex, mousey = love.mouse.getPosition()
		if mousex > v.x and mousex < v.x + v.font:getWidth(v.text) and mousey > v.y and mousey < v.y + v.font:getHeight(v.text) then
			v.hover = true
		else
			v.hover = false
		end
	end
end

function button_loadButtons()

	local Font = love.graphics.getFont()

	create("center",display.height-150,"New World",fonts.button,function() gamestate = "ChooseWorldType" end,"Main Menu")
	--create(display.width/2+25,display.height-150,"Load World",fonts.button,function()  end,"Main Menu")
	create("center",display.height-100,"Options",fonts.button,function() gamestate = "Options" end,"Main Menu")
	create("center",display.height-50,"Quit Game",fonts.button,function() love.event.quit() end,"Main Menu")

	create(0,0,"Back to Game",fonts.button,function() gamestate = "Editor" end,"Help")

	create(display.width/2-100,display.height-100,"Back to Game",fonts.button,function() editor.selected = "0"; player.doTick = true; gamestate = "Editor"; end,"Escape")
	create(display.width/2-100,display.height-50,"Quit to Menu",fonts.button,function() editor.selected = "0" gamestate = "Main Menu" end,"Escape")

	create(display.width/2-125,175,"Toggle Mute",fonts.button,function() if love.audio.getVolume() == 0 then love.audio.setVolume(1) else love.audio.setVolume(0) end end,"Options")
	create( "center", 250, "Controls", fonts.button, function() gamestate = "Controls"; controlChange.reset( ) end, "Options" )
	create( "center", 325, "Toggle Fullscreen", fonts.button, function() love.window.setFullscreen( not love.window.getFullscreen( ) ) end, "Options" )
	create(display.width/2-50,display.height-50,"Back",fonts.button,function() gamestate = "Main Menu" end,"Options")

	create( "center", display.height - 100, "Confirm", fonts.button, function() gamestate = "Options" controlChange.updateKeyBinds( ) end, "Controls" )
	create( "center", display.height - 50, "Cancel", fonts.button, function() gamestate = "Options" end, "Controls" )

	create( "center", 175, "Overworld", fonts.button2, function() world = bump.newWorld( 50 ); worldType = gentypes.overworld; editor.generate( gentypes.overworld ); end, "ChooseWorldType" )
	create( "center", 250, "Moon", fonts.button2, function() world = bump.newWorld( 50 ); worldType = gentypes.moon; editor.generate( gentypes.moon ); player.generateCollider( ); end, "ChooseWorldType" )
	create( "center", display.height - 100, "Back", fonts.button, function() gamestate = "Main Menu" end, "ChooseWorldType" )

	create( "center", 400, "Return to game", fonts.button, function() gamestate = "Editor"; player.generateCollider( ); player.x = ( editor.worldSize * editor.tdefsize ) / 2; player.doTick = true; player.y = 1000; player.yvel = 0; end, "Dead" )
	create( "center", 475, "Quit to menu", fonts.button, function() gamestate = "Main Menu" end, "Dead" )
end