editor = { }

editor.tdefsize = 50
editor.selected = 1
editor.worldSize = 300 --[=[ World size should not exceed 200. It's pretty laggy if it does ]=]--

editor.mousex = 0
editor.mousey = 0

editor.canReach = false

editor.mouseylimit = 20

worldCanvas = love.graphics.newCanvas( editor.worldSize * editor.tdefsize, editor.worldSize * editor.tdefsize )
worldCanvas:setFilter( "nearest", "nearest" )

local lclicked,  rclicked = false

function editor.drawMap( )
	worldCanvas:clear( )
	love.graphics.setCanvas( worldCanvas )
	for k, v in pairs( tiles ) do
		love.graphics.draw( blocks[ v.type ].texture, v.x, v.y )
	end
	love.graphics.setCanvas( )
end

function editor.draw( )
	love.graphics.setColour( 255, 255, 255 )
	love.graphics.draw( worldCanvas, 0, 0 )
	local f = love.graphics.getFont( )
	love.graphics.setFont( fonts.body )
	text.rightPrint( blockIDs[ editor.selected ].name .. " ( " .. editor.selected .. " )", 0, 10 )
	editor.drawX = ( love.mouse.getX( ) + camera.x )
	editor.drawY = ( love.mouse.getY( ) + camera.y )
	editor.mousex = math.ceil( editor.drawX / editor.tdefsize ) * editor.tdefsize - editor.tdefsize
	editor.mousey = math.ceil( editor.drawY / editor.tdefsize ) * editor.tdefsize - editor.tdefsize
	if editor.mousey > editor.mouseylimit+camera.y then
		if editor.selected ~= 0 then
			if editor.canReach == false then
				love.graphics.setColor( 0, 0, 0, 0 )
			else
				love.graphics.setColor( 50, 50, 50, 150 )
			end
			love.graphics.setLineWidth( 1 )
			love.graphics.rectangle( "line", editor.mousex, editor.mousey, editor.tdefsize, editor.tdefsize )
			love.graphics.setColor( 255, 255, 255, 255 )
		end
	end
end

function editor.moveCamera( dt )
	local k = love.keyboard
	if k.isDown( "lshift" ) then
		camera.speed = 1250
	elseif k.isDown( "lctrl" ) then
		camera.speed = 400
	else
		camera.speed = 750
	end
	if k.isDown( "up" ) then
		camera.y = camera.y - camera.speed * dt
	elseif k.isDown( "down" ) then
		camera.y = camera.y + camera.speed * dt
	end
	if k.isDown( "left" ) then
		camera.x = camera.x - camera.speed * dt
	elseif k.isDown( "right" ) then
		camera.x = camera.x + camera.speed * dt
	end
end

function editor.update( dt )
	--editor.moveCamera( dt )
	if math.abs( player.x - editor.mousex ) <= 75 and math.abs( player.y - editor.mousey ) <= 75 then
		editor.canReach = true
	else
		editor.canReach = false
	end

end

function editor.select( x, y, button )
	if button == "wu" then
		if blockIDs[ editor.selected + 1 ] == nil then
			editor.selected = 1
		else
			editor.selected = editor.selected + 1
		end
	elseif button == "wd" then
		if editor.selected > 1 then
			editor.selected = editor.selected - 1
		else
			editor.selected = #blockIDs
		end
	end
end

function editor.clicked( x, y, button )
	editor.select( x, y, button )
	if button == "l" and not rclicked then
		lclicked = true
	elseif button == "r" and not lclicked then
		rclicked = true
	end
end

function editor.unclicked( x, y )
	if rclicked and editor.canReach then --and player.inventory[ editor.selected ] > 0 
		if editor.mousey > editor.mouseylimit + camera.y then
			if editor.selected ~= 0 or texture[ editor.selected ] then
				local place = true
				for k, v in ipairs( tiles ) do
					if editor.mousex == v.x and editor.mousey == v.y then
						place = false
						break
					end
				end
				if place then
					local c = #tiles + 1
					tiles[ c ] = { 
						x = editor.mousex,
						y = editor.mousey,
						type = blockIDs[ editor.selected ].name
					}
					if blockIDs[ editor.selected ].hasColliders then
						world:add( tiles[ c ], editor.mousex, editor.mousey, editor.tdefsize, editor.tdefsize )
					end
				end
			end
		end
	elseif lclicked and editor.canReach then
		for k, v in ipairs( tiles ) do
			if editor.mousex == v.x and editor.mousey == v.y then
				if blocks[ v.type ].hasColliders then
					world:remove( v )
				end
				table.remove( tiles, k )
			end
		end
	end
	editor.drawMap( )
	lclicked = false
	rclicked = false
end

function editor.breakBlock( )

end

function editor.generate( type )
	
	editor.selected = 1

	physicsWorld = love.physics.newWorld( 0, worldType.gravity, true )
	player.generateCollider( )

	tiles = { }
	local xpos = 0
	local ypos = 3000

	local layers = type.layers
	local layerMin = type.layerMin
	local layerMax = type.layerMax
	local currentLayer = 1
	local layerStart = 0

	for _ = 1, editor.worldSize do
		for i = 1, #layers do
			for j = 1, love.math.random( type.layerMin[ i ], type.layerMax[ i ] ) do
				for k, v in ipairs( type.layers[ i ] ) do --GEN BLOCKS
					--if ypos - layerStart >= v.genMin and ypos - layerStart <= v.genMax then
						if love.math.random( 1, v.genRarity ) == 1 then
							local c = #tiles + 1
							tiles[ c ] = {
								x = xpos,
								y = ypos,
								type = v.name
							}
							if v.hasColliders then
								world:add( tiles[ c ], xpos, ypos, editor.tdefsize, editor.tdefsize )
							end
							ypos = ypos - editor.tdefsize
							break
						end
					--end
				end
				layerStart = ypos
			end
		end
		layerStart = 0
		ypos = 3000
		xpos = xpos + editor.tdefsize
	end


	editor.drawMap( )

	gamestate = "Editor"
end

function editor.save( )
	love.filesystem.setIdentity( defaultDir .. "/maps" )
	love.filesystem.write( editor.worldName, table.serialize( tiles, true, true ) )
	love.filesystem.setIdentity( defaultDir )
	editor.world = name
end

function editor.load( name )
	love.filesystem.setIdentity( defaultDir .. "/maps" )
	chunkMap = love.filesystem.load( name )
	tiles = chunkMap( )
	love.filesystem.setIdentity( defaultDir .. "/maps" )
	editor.world = name
end