require "libs.utils"
require "buttons"
require "libs.game"
require "setup"
require "camera"
require "editor"
require "tableSer"
require "map"
require "player"
require "blockManager"
require "genManager"
require "itemManager"
require "changeControls"

bump = require "bump"

--[[Code by Eric Bernard, 2015]]--

love.graphics.setColour = love.graphics.setColor
love.graphics.setBackgroundColour = love.graphics.setBackgroundColor
love.graphics.getBackgorundColour = love.graphics.getBackgroundColor
love.graphics.getColour = love.graphics.getColor

function love.load( )

	version = "Build 15.9.15_a"

	camera.x, camera.y = 0, 0

	setup.load( )

	loadItems( )
	loadBlocks( )
	loadGenTypes( )

	loadControlChanges( )

	worldType = gentypes.overworld

	time = 0
	timeCounter = 0
	timeOfDay = "Morning"

	love.graphics.setDefaultFilter( "nearest", "nearest" )

	debug = { collisions = false }

	button_loadButtons( )

	player.load( )

	gamestate = "Main Menu"
	textinput = ""

	for k, v in pairs( love.filesystem.getDirectoryItems( "/images/tiles" ) ) do
		if v ~= "Thumbs.db" then
			player.inventory[ v:sub( 1, #v-4 ) ] = 0
		end
	end

end



function love.update(dt)

	button_onHover(dt)
	game.update(dt)

	if gamestate == "Controls" then
		controlChange.update( dt )
	end

	if gamestate == "Editor" then
		player.update( dt )
	end
	
	timeCounter = timeCounter + dt
	if timeCounter > 0.1 then
		if timeOfDay == "Morning" then
			time = time - 0.1
		else
			time = time + 0.1
		end
		if time == 13 or time == -1 then
			if timeOfDay == "Morning" then
				timeOfDay = "Night"
				time = 0
			else
				timeOfDay = "Morning"
				time = 12
			end
		end
		timeCounter = 0
	end

	--music:play( )

end



function love.draw( )

	camera:set( )
	game.draw( )
	button_draw( )

	if gamestate == "Editor" then
		player.draw( dt )
	end

	--[=[if debug.collisions then
		for k, v in pairs( colliders ) do
			local r = { love.graphics.getColour( ) }
			love.graphics.setColour( 255, 0, 0 )
			love.graphics.rectangle( "line", v.x, v.y, editor.tdefsize, editor.tdefsize )
			love.graphics.setColour( unpack( r ) )
			--[=[ love.graphics.setFont( love.graphics.newFont( 15 ) )
			love.graphics.print( v.ID, v.x, v.y ) 
		end
	end--]=]
	if debug.collisions and gamestate == "Editor" then
		local items, l = world:getItems( )
		for k, v in pairs( items ) do
			local x, y, w, h = world:getRect( v )
			local c = { love.graphics.getColour( ) }
			love.graphics.setColour( 255, 0, 0 )
			love.graphics.rectangle( "line", x, y, w, h )
			love.graphics.setColour( unpack( c ) )
		end
	end

	if gamestate == "Controls" then
		controlChange.draw( )
	end

	camera:unset( )

end



function love.mousepressed(x,y,button) 

	button_onClick(x,y)
	editor.clicked(x,y,button)

end



function love.mousereleased(x,y)
	
	button_onRelease(x,y)
	if gamestate == "Editor" then
		editor.unclicked(x,y)
	end

end



function love.keypressed(key)
	
	if gamestate == "Controls" then
		controlChange.setKey( key )
	end

end

function love.keyreleased(key)

	if key == "escape" then
		if gamestate == "Editor" then
			gamestate = "Escape"
			player.doTick = false
		end
		if gamestate == "Game" then
			map.unload( )
			gamestate = "Editor"
		end
	end
	if key == "return" then
		if gamestate == "SaveWorldName" then
			editor.save(textinput)
			gamestate = "Editor"
			textinput = ""
			love.keyboard.setTextInput(false)
		elseif gamestate == "LoadWorldName" then
			editor.load(textinput)
			gamestate = "Editor" 
			textinput = ""
			love.keyboard.setTextInput(false)
		end
	end
	if key == controls.move.jump then
		player.canJump = true
	end
end

function love.textinput(t)

	if gamestate == "SaveWorldName" or gamestate == "LoadWorldName" then
		textinput = textinput..t
	end

end